@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2006-2013. All Rights Reserved.
+%% Copyright Ericsson AB 2006-2017. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -376,7 +376,7 @@ init_master(Parent,NodeOptsList,EvHandlers,MasterLogDir,LogDirs,
     end,
 
     %% start master event manager and add default handler
-    ct_master_event:start_link(),
+    {ok, _}  = start_ct_master_event(),
     ct_master_event:add_handler(),
     %% add user handlers for master event manager
     Add = fun({H,Args}) ->
@@ -398,6 +398,14 @@ init_master(Parent,NodeOptsList,EvHandlers,MasterLogDir,LogDirs,
     end,
     init_master1(Parent,NodeOptsList,InitOptions,LogDirs).
 
+start_ct_master_event() ->
+    case ct_master_event:start_link() of
+        {error, {already_started, Pid}} ->
+            {ok, Pid};
+        Else ->
+            Else
+    end.
+
 init_master1(Parent,NodeOptsList,InitOptions,LogDirs) ->
     {Inaccessible,NodeOptsList1,InitOptions1} = init_nodes(NodeOptsList,
 							   InitOptions),
@@ -426,7 +434,7 @@ init_master1(Parent,NodeOptsList,InitOptions,LogDirs) ->
 init_master2(Parent,NodeOptsList,LogDirs) ->
     process_flag(trap_exit,true),
     Cookie = erlang:get_cookie(),
-    log(all,"Cookie","~w",[Cookie]),
+    log(all,"Cookie","~tw",[Cookie]),
     log(all,"Starting Tests",
 	"Tests starting on: ~p",[[N || {N,_} <- NodeOptsList]]),
     SpawnAndMon = 
@@ -446,7 +454,7 @@ master_loop(#state{node_ctrl_pids=[],
 		   results=Finished}) ->
     Str =
 	lists:map(fun({Node,Result}) ->
-			  io_lib:format("~-40.40.*ts~p\n",
+			  io_lib:format("~-40.40.*ts~tp\n",
 					[$_,atom_to_list(Node),Result])
 		  end,lists:reverse(Finished)),
     log(all,"TEST RESULTS",Str,[]),
@@ -480,7 +488,7 @@ master_loop(State=#state{node_ctrl_pids=NodeCtrlPids,
 					Bad
 				end,
 			    log(all,"Test Info",
-				"Test on node ~w failed! Reason: ~p",
+				"Test on node ~w failed! Reason: ~tp",
 				[Node,Error]),
 			    {Locks1,Blocked1} = 
 				update_queue(exit,Node,Locks,Blocked),
@@ -493,7 +501,7 @@ master_loop(State=#state{node_ctrl_pids=NodeCtrlPids,
 		undefined ->
 		    %% ignore (but report) exit from master_logger etc
 		    log(all,"Test Info",
-			"Warning! Process ~w has terminated. Reason: ~p",
+			"Warning! Process ~w has terminated. Reason: ~tp",
 			[Pid,Reason]),
 			master_loop(State)
 	    end;
@@ -576,7 +584,7 @@ update_queue(take,Node,From,Lock={Op,Resource},Locks,Blocked) ->
     %% Blocked: [{{Operation,Resource},Node,WaitingPid},...]
     case lists:keysearch(Lock,1,Locks) of
 	{value,{_Lock,Owner}} ->		% other node has lock
-	    log(html,"Lock Info","Node ~w blocked on ~w by ~w. Resource: ~p",
+	    log(html,"Lock Info","Node ~w blocked on ~w by ~w. Resource: ~tp",
 		[Node,Op,Owner,Resource]),
 	    Blocked1 = Blocked ++ [{Lock,Node,From}],
 	    {Locks,Blocked1};
@@ -591,7 +599,7 @@ update_queue(release,Node,_From,Lock={Op,Resource},Locks,Blocked) ->
     case lists:keysearch(Lock,1,Blocked) of
 	{value,E={Lock,SomeNode,WaitingPid}} ->
 	    Blocked1 = lists:delete(E,Blocked),
-	    log(html,"Lock Info","Node ~w proceeds with ~w. Resource: ~p",
+	    log(html,"Lock Info","Node ~w proceeds with ~w. Resource: ~tp",
 		[SomeNode,Op,Resource]),
 	    reply(ok,WaitingPid),		% waiting process may start
 	    {Locks1,Blocked1};
@@ -658,7 +666,7 @@ refresh_logs([D|Dirs],Refreshed) ->
 		    {ok,Cwd} = file:get_cwd(),
 		    case catch ct_run:refresh_logs(D) of
 			{'EXIT',Reason} ->
-			    file:set_cwd(Cwd),
+			    ok = file:set_cwd(Cwd),
 			    refresh_logs(Dirs,[{D,{error,Reason}}|Refreshed]);
 			Result -> 
 			    refresh_logs(Dirs,[{D,Result}|Refreshed])
@@ -670,7 +678,7 @@ refresh_logs([D|Dirs],Refreshed) ->
 refresh_logs([],Refreshed) ->
     Str =
 	lists:map(fun({D,Result}) ->
-			  io_lib:format("Refreshing logs in ~p... ~p",
+			  io_lib:format("Refreshing logs in ~tp... ~tp",
 					[D,Result])
 		  end,Refreshed),
     log(all,"Info",Str,[]).
@@ -701,10 +709,10 @@ init_node_ctrl(MasterPid,Cookie,Opts) ->
     end,
     
     %% start a local event manager
-    ct_event:start_link(),
+    {ok, _} = start_ct_event(),
     ct_event:add_handler([{master,MasterPid}]),
 
-    %% log("Running test with options: ~p~n", [Opts]),
+    %% log("Running test with options: ~tp~n", [Opts]),
     Result = case (catch ct:run_test(Opts)) of
 		 ok -> finished_ok;
 		 Other -> Other
@@ -721,6 +729,14 @@ init_node_ctrl(MasterPid,Cookie,Opts) ->
 		      "Can't report result!~n~n", [MasterNode])
     end.
 
+start_ct_event() ->
+    case ct_event:start_link() of
+        {error, {already_started, Pid}} ->
+            {ok, Pid};
+        Else ->
+            Else
+    end.
+
 %%%-----------------------------------------------------------------
 %%% Event handling
 %%%-----------------------------------------------------------------
@@ -778,7 +794,7 @@ reply(Result,To) ->
     ok.
 
 init_nodes(NodeOptions, InitOptions)->
-    ping_nodes(NodeOptions),
+    _ = ping_nodes(NodeOptions),
     start_nodes(InitOptions),
     eval_on_nodes(InitOptions),
     {Inaccessible, NodeOptions1}=ping_nodes(NodeOptions),
@@ -812,7 +828,7 @@ start_nodes(InitOptions)->
 				  "with callback ~w~n", [NodeName,Callback]);
 		    {error, Reason, _NodeName} ->
 			io:format("Failed to start node ~w with callback ~w! "
-				  "Reason: ~p~n", [NodeName, Callback, Reason])
+				  "Reason: ~tp~n", [NodeName, Callback, Reason])
 		end;
 	    {true, true}->
 		io:format("WARNING: Node ~w is alive but has node_start "
@@ -841,10 +857,10 @@ eval_on_nodes(InitOptions)->
 evaluate(Node, [{M,F,A}|MFAs])->
     case rpc:call(Node, M, F, A) of
         {badrpc,Reason}->
-	    io:format("WARNING: Failed to call ~w:~w/~w on node ~w "
-		      "due to ~p~n", [M,F,length(A),Node,Reason]);
+	    io:format("WARNING: Failed to call ~w:~tw/~w on node ~w "
+		      "due to ~tp~n", [M,F,length(A),Node,Reason]);
 	Result->
-	    io:format("Called ~w:~w/~w on node ~w, result: ~p~n",
+	    io:format("Called ~w:~tw/~w on node ~w, result: ~tp~n",
 		      [M,F,length(A),Node,Result])
     end,
     evaluate(Node, MFAs);
