@@ -201,7 +201,7 @@ name_it(Name) -> Name.
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2015. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2017. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -351,27 +351,26 @@ yecctoken_location(Token) ->
     end.
 
 -compile({nowarn_unused_function, yecctoken2string/1}).
-yecctoken2string({atom, _, A}) -> io_lib:write(A);
+yecctoken2string({atom, _, A}) -> io_lib:write_atom(A);
 yecctoken2string({integer,_,N}) -> io_lib:write(N);
 yecctoken2string({float,_,F}) -> io_lib:write(F);
 yecctoken2string({char,_,C}) -> io_lib:write_char(C);
 yecctoken2string({var,_,V}) -> io_lib:format("~s", [V]);
 yecctoken2string({string,_,S}) -> io_lib:write_string(S);
 yecctoken2string({reserved_symbol, _, A}) -> io_lib:write(A);
-yecctoken2string({_Cat, _, Val}) -> io_lib:format("~p",[Val]);
+yecctoken2string({_Cat, _, Val}) -> io_lib:format("~tp", [Val]);
 yecctoken2string({dot, _}) -> "'.'";
-yecctoken2string({'$end', _}) ->
-    [];
+yecctoken2string({'$end', _}) -> [];
 yecctoken2string({Other, _}) when is_atom(Other) ->
-    io_lib:write(Other);
+    io_lib:write_atom(Other);
 yecctoken2string(Other) ->
-    io_lib:write(Other).
+    io_lib:format("~tp", [Other]).
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
 
--file("xref_parser.erl", 374).
+-file("xref_parser.erl", 373).
 
 -dialyzer({nowarn_function, yeccpars2/7}).
 yeccpars2(0=S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -573,28 +572,28 @@ yeccpars2_2(S, ':=', Ss, Stack, T, Ts, Tzr) ->
 yeccpars2_2(S, '=', Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 78, Ss, Stack, T, Ts, Tzr);
 yeccpars2_2(_S, '$end', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_2_$end'(Stack),
+ NewStack = 'yeccpars2_2_\'$end\''(Stack),
  yeccgoto_expr(hd(Ss), '$end', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_2(_S, '*', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_2_*'(Stack),
+ NewStack = 'yeccpars2_2_\'*\''(Stack),
  yeccgoto_expr(hd(Ss), '*', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_2(_S, '+', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_2_+'(Stack),
+ NewStack = 'yeccpars2_2_\'+\''(Stack),
  yeccgoto_expr(hd(Ss), '+', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_2(_S, '-', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_2_-'(Stack),
+ NewStack = 'yeccpars2_2_\'-\''(Stack),
  yeccgoto_expr(hd(Ss), '-', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_2(_S, 'of', Ss, Stack, T, Ts, Tzr) ->
- NewStack = yeccpars2_2_of(Stack),
+ NewStack = 'yeccpars2_2_\'of\''(Stack),
  yeccgoto_expr(hd(Ss), 'of', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_2(_S, '|', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_2_|'(Stack),
+ NewStack = 'yeccpars2_2_\'|\''(Stack),
  yeccgoto_expr(hd(Ss), '|', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_2(_S, '||', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_2_||'(Stack),
+ NewStack = 'yeccpars2_2_\'||\''(Stack),
  yeccgoto_expr(hd(Ss), '||', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_2(_S, '|||', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_2_|||'(Stack),
+ NewStack = 'yeccpars2_2_\'|||\''(Stack),
  yeccgoto_expr(hd(Ss), '|||', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_2(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_2_(Stack),
@@ -615,34 +614,34 @@ yeccpars2_5(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccpars2_6(S, decl, Ss, Stack, T, Ts, Tzr) ->
  yeccpars1(S, 33, Ss, Stack, T, Ts, Tzr);
 yeccpars2_6(_S, '$end', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_6_$end'(Stack),
+ NewStack = 'yeccpars2_6_\'$end\''(Stack),
  yeccpars2_73(_S, '$end', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, ')', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_6_)'(Stack),
+ NewStack = 'yeccpars2_6_\')\''(Stack),
  yeccpars2_73(_S, ')', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, '*', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_6_*'(Stack),
+ NewStack = 'yeccpars2_6_\'*\''(Stack),
  yeccpars2_73(_S, '*', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, '+', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_6_+'(Stack),
+ NewStack = 'yeccpars2_6_\'+\''(Stack),
  yeccpars2_73(_S, '+', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, ',', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_6_,'(Stack),
+ NewStack = 'yeccpars2_6_\',\''(Stack),
  yeccpars2_73(_S, ',', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, '-', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_6_-'(Stack),
+ NewStack = 'yeccpars2_6_\'-\''(Stack),
  yeccpars2_73(_S, '-', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, 'of', Ss, Stack, T, Ts, Tzr) ->
- NewStack = yeccpars2_6_of(Stack),
+ NewStack = 'yeccpars2_6_\'of\''(Stack),
  yeccpars2_73(_S, 'of', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, '|', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_6_|'(Stack),
+ NewStack = 'yeccpars2_6_\'|\''(Stack),
  yeccpars2_73(_S, '|', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, '||', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_6_||'(Stack),
+ NewStack = 'yeccpars2_6_\'||\''(Stack),
  yeccpars2_73(_S, '||', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, '|||', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_6_|||'(Stack),
+ NewStack = 'yeccpars2_6_\'|||\''(Stack),
  yeccpars2_73(_S, '|||', [6 | Ss], NewStack, T, Ts, Tzr);
 yeccpars2_6(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccgoto_regatom(hd(Ss), Cat, Ss, Stack, T, Ts, Tzr).
@@ -677,37 +676,37 @@ yeccpars2_10(_S, vertex, Ss, Stack, T, Ts, Tzr) ->
 yeccpars2_10(_S, '{', Ss, Stack, T, Ts, Tzr) ->
  yeccgoto_prefix_op(hd(Ss), '{', Ss, Stack, T, Ts, Tzr);
 yeccpars2_10(_S, '$end', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_10_$end'(Stack),
+ NewStack = 'yeccpars2_10_\'$end\''(Stack),
  yeccgoto_const(hd(Ss), '$end', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, ')', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_10_)'(Stack),
+ NewStack = 'yeccpars2_10_\')\''(Stack),
  yeccgoto_const(hd(Ss), ')', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, '*', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_10_*'(Stack),
+ NewStack = 'yeccpars2_10_\'*\''(Stack),
  yeccgoto_const(hd(Ss), '*', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, '+', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_10_+'(Stack),
+ NewStack = 'yeccpars2_10_\'+\''(Stack),
  yeccgoto_const(hd(Ss), '+', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, ',', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_10_,'(Stack),
+ NewStack = 'yeccpars2_10_\',\''(Stack),
  yeccgoto_const(hd(Ss), ',', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, '-', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_10_-'(Stack),
+ NewStack = 'yeccpars2_10_\'-\''(Stack),
  yeccgoto_const(hd(Ss), '-', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, decl, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_10_decl(Stack),
  yeccgoto_const(hd(Ss), decl, Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, 'of', Ss, Stack, T, Ts, Tzr) ->
- NewStack = yeccpars2_10_of(Stack),
+ NewStack = 'yeccpars2_10_\'of\''(Stack),
  yeccgoto_const(hd(Ss), 'of', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, '|', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_10_|'(Stack),
+ NewStack = 'yeccpars2_10_\'|\''(Stack),
  yeccgoto_const(hd(Ss), '|', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, '||', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_10_||'(Stack),
+ NewStack = 'yeccpars2_10_\'||\''(Stack),
  yeccgoto_const(hd(Ss), '||', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, '|||', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_10_|||'(Stack),
+ NewStack = 'yeccpars2_10_\'|||\''(Stack),
  yeccgoto_const(hd(Ss), '|||', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_10(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_10_(Stack),
@@ -867,34 +866,34 @@ yeccpars2_37(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccgoto_expr(hd(Nss), Cat, Nss, NewStack, T, Ts, Tzr).
 
 yeccpars2_38(_S, '$end', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_38_$end'(Stack),
+ NewStack = 'yeccpars2_38_\'$end\''(Stack),
  yeccgoto_expr(hd(Ss), '$end', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_38(_S, ')', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_38_)'(Stack),
+ NewStack = 'yeccpars2_38_\')\''(Stack),
  yeccgoto_expr(hd(Ss), ')', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_38(_S, '*', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_38_*'(Stack),
+ NewStack = 'yeccpars2_38_\'*\''(Stack),
  yeccgoto_expr(hd(Ss), '*', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_38(_S, '+', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_38_+'(Stack),
+ NewStack = 'yeccpars2_38_\'+\''(Stack),
  yeccgoto_expr(hd(Ss), '+', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_38(_S, ',', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_38_,'(Stack),
+ NewStack = 'yeccpars2_38_\',\''(Stack),
  yeccgoto_expr(hd(Ss), ',', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_38(_S, '-', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_38_-'(Stack),
+ NewStack = 'yeccpars2_38_\'-\''(Stack),
  yeccgoto_expr(hd(Ss), '-', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_38(_S, 'of', Ss, Stack, T, Ts, Tzr) ->
- NewStack = yeccpars2_38_of(Stack),
+ NewStack = 'yeccpars2_38_\'of\''(Stack),
  yeccgoto_expr(hd(Ss), 'of', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_38(_S, '|', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_38_|'(Stack),
+ NewStack = 'yeccpars2_38_\'|\''(Stack),
  yeccgoto_expr(hd(Ss), '|', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_38(_S, '||', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_38_||'(Stack),
+ NewStack = 'yeccpars2_38_\'||\''(Stack),
  yeccgoto_expr(hd(Ss), '||', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_38(_S, '|||', Ss, Stack, T, Ts, Tzr) ->
- NewStack = 'yeccpars2_38_|||'(Stack),
+ NewStack = 'yeccpars2_38_\'|||\''(Stack),
  yeccgoto_expr(hd(Ss), '|||', Ss, NewStack, T, Ts, Tzr);
 yeccpars2_38(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_38_(Stack),
@@ -1129,6 +1128,7 @@ yeccpars2_79(_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  NewStack = yeccpars2_79_(Stack),
  yeccgoto_statement(hd(Nss), Cat, Nss, NewStack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_add_op/7}).
 yeccgoto_add_op(11, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(45, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_add_op(39, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1150,9 +1150,11 @@ yeccgoto_add_op(61, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_add_op(79, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(45, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_assign_op/7}).
 yeccgoto_assign_op(2, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(76, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_cast_op/7}).
 yeccgoto_cast_op(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(15, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_cast_op(9, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1176,6 +1178,7 @@ yeccgoto_cast_op(74, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_cast_op(76, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(15, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_const/7}).
 yeccgoto_const(0=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_14(_S, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_const(9=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1205,6 +1208,7 @@ yeccgoto_const(74=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_const(76=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_14(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_constant/7}).
 yeccgoto_constant(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_13(13, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_constant(9, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1234,6 +1238,7 @@ yeccgoto_constant(74, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_constant(76, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_13(13, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_constants/7}).
 yeccgoto_constants(26, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_27(27, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_constants(29=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1241,6 +1246,7 @@ yeccgoto_constants(29=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_constants(34, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_35(35, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_count_op/7}).
 yeccgoto_count_op(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(12, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_count_op(9, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1264,6 +1270,7 @@ yeccgoto_count_op(74, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_count_op(76, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(12, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_expr/7}).
 yeccgoto_expr(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_11(11, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_expr(9=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1287,6 +1294,7 @@ yeccgoto_expr(74, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_expr(76, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_79(79, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_id/7}).
 yeccgoto_id(0=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_10(_S, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_id(9=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1318,6 +1326,7 @@ yeccgoto_id(74=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_id(76=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_10(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_mult_op/7}).
 yeccgoto_mult_op(11, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(44, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_mult_op(39, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1339,6 +1348,7 @@ yeccgoto_mult_op(61, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_mult_op(79, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(44, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_path_op/7}).
 yeccgoto_path_op(11, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(43, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_path_op(39, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1360,6 +1370,7 @@ yeccgoto_path_op(61, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_path_op(79, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(43, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_prefix_op/7}).
 yeccgoto_prefix_op(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(9, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_prefix_op(9, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1383,6 +1394,7 @@ yeccgoto_prefix_op(74, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_prefix_op(76, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(9, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_regatom/7}).
 yeccgoto_regatom(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_8(8, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_regatom(9, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1408,6 +1420,7 @@ yeccgoto_regatom(74, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_regatom(76, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_8(8, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_regexp/7}).
 yeccgoto_regexp(0=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_7(_S, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_regexp(9=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1431,9 +1444,11 @@ yeccgoto_regexp(74=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_regexp(76=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_7(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_regint/7}).
 yeccgoto_regint(67, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_70(70, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_regstr/7}).
 yeccgoto_regstr(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_6(6, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_regstr(9, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1461,6 +1476,7 @@ yeccgoto_regstr(74, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_regstr(76, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_6(6, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_regvar/7}).
 yeccgoto_regvar(0=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_5(_S, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_regvar(9=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1488,6 +1504,7 @@ yeccgoto_regvar(74=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_regvar(76=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_5(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_restr_op/7}).
 yeccgoto_restr_op(11, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(42, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_restr_op(39, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1509,16 +1526,19 @@ yeccgoto_restr_op(61, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_restr_op(79, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_0(42, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_statement/7}).
 yeccgoto_statement(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_4(4, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_statement(74, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_4(4, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_statements/7}).
 yeccgoto_statements(0=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_3(_S, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_statements(74=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_75(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_type/7}).
 yeccgoto_type(6=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_73(_S, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_type(13=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1530,6 +1550,7 @@ yeccgoto_type(36=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_type(70=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_72(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_variable/7}).
 yeccgoto_variable(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_2(2, Cat, Ss, Stack, T, Ts, Tzr);
 yeccgoto_variable(9=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
@@ -1557,68 +1578,69 @@ yeccgoto_variable(74, Cat, Ss, Stack, T, Ts, Tzr) ->
 yeccgoto_variable(76=_S, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_38(_S, Cat, Ss, Stack, T, Ts, Tzr).
 
+-dialyzer({nowarn_function, yeccgoto_xref/7}).
 yeccgoto_xref(0, Cat, Ss, Stack, T, Ts, Tzr) ->
  yeccpars2_1(1, Cat, Ss, Stack, T, Ts, Tzr).
 
--compile({inline,'yeccpars2_2_$end'/1}).
+-compile({inline,'yeccpars2_2_\'$end\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_2_$end'(__Stack0) ->
+'yeccpars2_2_\'$end\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_2_*'/1}).
+-compile({inline,'yeccpars2_2_\'*\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_2_*'(__Stack0) ->
+'yeccpars2_2_\'*\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_2_+'/1}).
+-compile({inline,'yeccpars2_2_\'+\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_2_+'(__Stack0) ->
+'yeccpars2_2_\'+\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_2_-'/1}).
+-compile({inline,'yeccpars2_2_\'-\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_2_-'(__Stack0) ->
+'yeccpars2_2_\'-\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,yeccpars2_2_of/1}).
+-compile({inline,'yeccpars2_2_\'of\''/1}).
 -file("xref_parser.yrl", 63).
-yeccpars2_2_of(__Stack0) ->
+'yeccpars2_2_\'of\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_2_|'/1}).
+-compile({inline,'yeccpars2_2_\'|\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_2_|'(__Stack0) ->
+'yeccpars2_2_\'|\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_2_||'/1}).
+-compile({inline,'yeccpars2_2_\'||\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_2_||'(__Stack0) ->
+'yeccpars2_2_\'||\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_2_|||'/1}).
+-compile({inline,'yeccpars2_2_\'|||\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_2_|||'(__Stack0) ->
+'yeccpars2_2_\'|||\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
@@ -1640,119 +1662,119 @@ yeccpars2_4_(__Stack0) ->
    [ __1 ]
   end | __Stack].
 
--compile({inline,'yeccpars2_6_$end'/1}).
+-compile({inline,'yeccpars2_6_\'$end\''/1}).
 -file("xref_parser.yrl", 102).
-'yeccpars2_6_$end'(__Stack0) ->
+'yeccpars2_6_\'$end\''(__Stack0) ->
  [begin
    unknown
   end | __Stack0].
 
--compile({inline,'yeccpars2_6_)'/1}).
+-compile({inline,'yeccpars2_6_\')\''/1}).
 -file("xref_parser.yrl", 102).
-'yeccpars2_6_)'(__Stack0) ->
+'yeccpars2_6_\')\''(__Stack0) ->
  [begin
    unknown
   end | __Stack0].
 
--compile({inline,'yeccpars2_6_*'/1}).
+-compile({inline,'yeccpars2_6_\'*\''/1}).
 -file("xref_parser.yrl", 102).
-'yeccpars2_6_*'(__Stack0) ->
+'yeccpars2_6_\'*\''(__Stack0) ->
  [begin
    unknown
   end | __Stack0].
 
--compile({inline,'yeccpars2_6_+'/1}).
+-compile({inline,'yeccpars2_6_\'+\''/1}).
 -file("xref_parser.yrl", 102).
-'yeccpars2_6_+'(__Stack0) ->
+'yeccpars2_6_\'+\''(__Stack0) ->
  [begin
    unknown
   end | __Stack0].
 
--compile({inline,'yeccpars2_6_,'/1}).
+-compile({inline,'yeccpars2_6_\',\''/1}).
 -file("xref_parser.yrl", 102).
-'yeccpars2_6_,'(__Stack0) ->
+'yeccpars2_6_\',\''(__Stack0) ->
  [begin
    unknown
   end | __Stack0].
 
--compile({inline,'yeccpars2_6_-'/1}).
+-compile({inline,'yeccpars2_6_\'-\''/1}).
 -file("xref_parser.yrl", 102).
-'yeccpars2_6_-'(__Stack0) ->
+'yeccpars2_6_\'-\''(__Stack0) ->
  [begin
    unknown
   end | __Stack0].
 
--compile({inline,yeccpars2_6_of/1}).
+-compile({inline,'yeccpars2_6_\'of\''/1}).
 -file("xref_parser.yrl", 102).
-yeccpars2_6_of(__Stack0) ->
+'yeccpars2_6_\'of\''(__Stack0) ->
  [begin
    unknown
   end | __Stack0].
 
--compile({inline,'yeccpars2_6_|'/1}).
+-compile({inline,'yeccpars2_6_\'|\''/1}).
 -file("xref_parser.yrl", 102).
-'yeccpars2_6_|'(__Stack0) ->
+'yeccpars2_6_\'|\''(__Stack0) ->
  [begin
    unknown
   end | __Stack0].
 
--compile({inline,'yeccpars2_6_||'/1}).
+-compile({inline,'yeccpars2_6_\'||\''/1}).
 -file("xref_parser.yrl", 102).
-'yeccpars2_6_||'(__Stack0) ->
+'yeccpars2_6_\'||\''(__Stack0) ->
  [begin
    unknown
   end | __Stack0].
 
--compile({inline,'yeccpars2_6_|||'/1}).
+-compile({inline,'yeccpars2_6_\'|||\''/1}).
 -file("xref_parser.yrl", 102).
-'yeccpars2_6_|||'(__Stack0) ->
+'yeccpars2_6_\'|||\''(__Stack0) ->
  [begin
    unknown
   end | __Stack0].
 
--compile({inline,'yeccpars2_10_$end'/1}).
+-compile({inline,'yeccpars2_10_\'$end\''/1}).
 -file("xref_parser.yrl", 79).
-'yeccpars2_10_$end'(__Stack0) ->
+'yeccpars2_10_\'$end\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { constant , unknown , vertex , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_10_)'/1}).
+-compile({inline,'yeccpars2_10_\')\''/1}).
 -file("xref_parser.yrl", 79).
-'yeccpars2_10_)'(__Stack0) ->
+'yeccpars2_10_\')\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { constant , unknown , vertex , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_10_*'/1}).
+-compile({inline,'yeccpars2_10_\'*\''/1}).
 -file("xref_parser.yrl", 79).
-'yeccpars2_10_*'(__Stack0) ->
+'yeccpars2_10_\'*\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { constant , unknown , vertex , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_10_+'/1}).
+-compile({inline,'yeccpars2_10_\'+\''/1}).
 -file("xref_parser.yrl", 79).
-'yeccpars2_10_+'(__Stack0) ->
+'yeccpars2_10_\'+\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { constant , unknown , vertex , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_10_,'/1}).
+-compile({inline,'yeccpars2_10_\',\''/1}).
 -file("xref_parser.yrl", 79).
-'yeccpars2_10_,'(__Stack0) ->
+'yeccpars2_10_\',\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { constant , unknown , vertex , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_10_-'/1}).
+-compile({inline,'yeccpars2_10_\'-\''/1}).
 -file("xref_parser.yrl", 79).
-'yeccpars2_10_-'(__Stack0) ->
+'yeccpars2_10_\'-\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { constant , unknown , vertex , __1 }
@@ -1766,33 +1788,33 @@ yeccpars2_10_decl(__Stack0) ->
    { constant , unknown , vertex , __1 }
   end | __Stack].
 
--compile({inline,yeccpars2_10_of/1}).
+-compile({inline,'yeccpars2_10_\'of\''/1}).
 -file("xref_parser.yrl", 79).
-yeccpars2_10_of(__Stack0) ->
+'yeccpars2_10_\'of\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { constant , unknown , vertex , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_10_|'/1}).
+-compile({inline,'yeccpars2_10_\'|\''/1}).
 -file("xref_parser.yrl", 79).
-'yeccpars2_10_|'(__Stack0) ->
+'yeccpars2_10_\'|\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { constant , unknown , vertex , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_10_||'/1}).
+-compile({inline,'yeccpars2_10_\'||\''/1}).
 -file("xref_parser.yrl", 79).
-'yeccpars2_10_||'(__Stack0) ->
+'yeccpars2_10_\'||\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { constant , unknown , vertex , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_10_|||'/1}).
+-compile({inline,'yeccpars2_10_\'|||\''/1}).
 -file("xref_parser.yrl", 79).
-'yeccpars2_10_|||'(__Stack0) ->
+'yeccpars2_10_\'|||\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { constant , unknown , vertex , __1 }
@@ -1944,81 +1966,81 @@ yeccpars2_37_(__Stack0) ->
    type ( { list , [ __2 | __3 ] } , __5 )
   end | __Stack].
 
--compile({inline,'yeccpars2_38_$end'/1}).
+-compile({inline,'yeccpars2_38_\'$end\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_38_$end'(__Stack0) ->
+'yeccpars2_38_\'$end\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_38_)'/1}).
+-compile({inline,'yeccpars2_38_\')\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_38_)'(__Stack0) ->
+'yeccpars2_38_\')\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_38_*'/1}).
+-compile({inline,'yeccpars2_38_\'*\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_38_*'(__Stack0) ->
+'yeccpars2_38_\'*\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_38_+'/1}).
+-compile({inline,'yeccpars2_38_\'+\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_38_+'(__Stack0) ->
+'yeccpars2_38_\'+\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_38_,'/1}).
+-compile({inline,'yeccpars2_38_\',\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_38_,'(__Stack0) ->
+'yeccpars2_38_\',\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_38_-'/1}).
+-compile({inline,'yeccpars2_38_\'-\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_38_-'(__Stack0) ->
+'yeccpars2_38_\'-\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,yeccpars2_38_of/1}).
+-compile({inline,'yeccpars2_38_\'of\''/1}).
 -file("xref_parser.yrl", 63).
-yeccpars2_38_of(__Stack0) ->
+'yeccpars2_38_\'of\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_38_|'/1}).
+-compile({inline,'yeccpars2_38_\'|\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_38_|'(__Stack0) ->
+'yeccpars2_38_\'|\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_38_||'/1}).
+-compile({inline,'yeccpars2_38_\'||\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_38_||'(__Stack0) ->
+'yeccpars2_38_\'||\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
   end | __Stack].
 
--compile({inline,'yeccpars2_38_|||'/1}).
+-compile({inline,'yeccpars2_38_\'|||\''/1}).
 -file("xref_parser.yrl", 63).
-'yeccpars2_38_|||'(__Stack0) ->
+'yeccpars2_38_\'|||\''(__Stack0) ->
  [__1 | __Stack] = __Stack0,
  [begin
    { variable , __1 }
